@@ -1745,14 +1745,18 @@ static void failover(void)
 	/* failback request? */
 	if (Req_info->kind == NODE_UP_REQUEST)
 	{
-		if (node_id >= MAX_NUM_BACKENDS ||
+		if (node_id < 0 || node_id >= MAX_NUM_BACKENDS ||
 			(Req_info->kind == NODE_UP_REQUEST && !(RAW_MODE &&
             BACKEND_INFO(node_id).backend_status == CON_DOWN) && VALID_BACKEND(node_id)) ||
 			(Req_info->kind == NODE_DOWN_REQUEST && !VALID_BACKEND(node_id)))
 		{
 			pool_semaphore_unlock(REQUEST_INFO_SEM);
-			pool_error("failover_handler: invalid node_id %d status:%d MAX_NUM_BACKENDS: %d", node_id,
-					   BACKEND_INFO(node_id).backend_status, MAX_NUM_BACKENDS);
+
+			if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
+				pool_error("failover_handler: invalid node_id %d MAX_NUM_BACKENDS: %d", node_id, MAX_NUM_BACKENDS);
+			else
+				pool_error("failover_handler: invalid node_id %d status:%d MAX_NUM_BACKENDS: %d", node_id,
+						   BACKEND_INFO(node_id).backend_status, MAX_NUM_BACKENDS);
 			kill(pcp_pid, SIGUSR2);
 			switching = 0;
 			Req_info->switching = false;
@@ -1925,6 +1929,21 @@ static void failover(void)
 
 	if (Req_info->kind == PROMOTE_NODE_REQUEST && VALID_BACKEND(node_id))
 		new_primary = node_id;
+
+	/*
+	 * If the down node was a standby node in streaming replication
+	 * mode, we can avoid calling find_primary_node_repeatedly() and
+	 * recognize the former primary as the new primary node, which
+	 * will reduce the time to process standby down.
+	 */
+	else if (MASTER_SLAVE && !strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+			 Req_info->kind == NODE_DOWN_REQUEST)
+	{
+		if (Req_info->primary_node_id != node_id)
+			new_primary = Req_info->primary_node_id;
+		else
+			new_primary =  find_primary_node_repeatedly();
+	}
 	else
 		new_primary =  find_primary_node_repeatedly();
 
