@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2012	PgPool Global Development Group
+ * Copyright (c) 2003-2013	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -722,7 +722,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 		/*
 		 * Add bind message's info to query to search.
 		 */
-		if (bind_msg->param_offset && bind_msg->contents)
+		if (query_context->is_cache_safe && bind_msg->param_offset && bind_msg->contents)
 		{
 			/* Extract binary contents from bind message */
 			char *query_in_bind_msg = bind_msg->contents + bind_msg->param_offset;
@@ -745,7 +745,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 				snprintf(hex_str, sizeof(hex_str), (i == 0) ? " %02X" : "%02X", 0xff & query_in_bind_msg[i]);
 				hexlen = strlen(hex_str);
 
-				if ((len+hexlen) > alloc_len)
+				if ((len+hexlen) >= alloc_len)
 				{
 					alloc_len += STR_ALLOC_SIZE;
 					search_query = realloc(search_query, alloc_len);
@@ -1247,6 +1247,7 @@ POOL_STATUS Bind(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 	POOL_SENT_MESSAGE *bind_msg;
 	POOL_SESSION_CONTEXT *session_context;
 	POOL_QUERY_CONTEXT *query_context;
+	int insert_stmt_with_lock = 0;
 
 	/* Get session context */
 	session_context = pool_get_session_context();
@@ -1292,7 +1293,7 @@ POOL_STATUS Bind(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 	 */
 	if (query_context->is_cache_safe)
 	{
-		bind_msg->param_offset = sizeof(int) + sizeof(char) * (strlen(portal_name) + strlen(pstmt_name));
+		bind_msg->param_offset = sizeof(char) * (strlen(portal_name) + strlen(pstmt_name) + 2);
 	}
 
 	session_context->uncompleted_message = bind_msg;
@@ -1316,26 +1317,43 @@ POOL_STATUS Bind(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 			return POOL_END;
 	}
 
+	/*
+	 * Start a transaction if necessary
+	 */
+	pool_debug("Bind: checking strict query");
+	if (is_strict_query(query_context->parse_tree))
+	{
+		pool_debug("Bind: strict query");
+		start_internal_transaction(frontend, backend, query_context->parse_tree);
+		allow_close_transaction = 1;
+	}
+
+	pool_debug("Bind: checking insert lock");
+	insert_stmt_with_lock = need_insert_lock(backend, query_context->original_query, query_context->parse_tree);
+	if (insert_stmt_with_lock)
+	{
+		pool_debug("Bind: issuing insert lock");
+		/* issue a LOCK command to keep consistency */
+		if (insert_lock(frontend, backend, query_context->original_query, (InsertStmt *)query_context->parse_tree, insert_stmt_with_lock) != POOL_CONTINUE)
+		{
+			pool_query_context_destroy(query_context);
+			return POOL_END;
+		}
+	}
+
 	pool_debug("Bind: waiting for master completing the query");
 	if (pool_extended_send_and_wait(query_context, "B", len, contents, 1, MASTER_NODE_ID)
 		!= POOL_CONTINUE)
 	{
-		if (rewrite_msg != NULL)
-			/* free(rewrite_msg) */;
 		return POOL_END;
 	}
 
 	if (pool_extended_send_and_wait(query_context, "B", len, contents, -1, MASTER_NODE_ID)
 		!= POOL_CONTINUE)
 	{
-		if (rewrite_msg != NULL)
-			/* free(rewrite_msg) */;
 		return POOL_END;
 	}
 
-	if (rewrite_msg != NULL)
-		/* free(rewrite_msg) */;
-
 	return POOL_CONTINUE;
 }
 
@@ -1533,7 +1551,6 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 	if (session_context->mismatch_ntuples && MAJOR(backend) == PROTO_MAJOR_V3)
 	{
 		int i;
-		signed char state;
 		char kind;
 
 		/*
@@ -1808,6 +1825,10 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 				}
 				else
 				{
+					if (MAJOR(backend) != PROTO_MAJOR_V3)
+					{
+						state = 'I';	/* XXX I don't think query cache works with PROTO2 protocol */
+					}
 					pool_handle_query_cache(backend, query, node, state);
 				}
 			}
@@ -2916,8 +2937,9 @@ POOL_STATUS CopyDataRows(POOL_CONNECTION *frontend,
 						id = pool_get_id(info, p1);
 						pool_debug("CopyDataRow: copying id: %d", id);
 						free(p1);
-						if (!VALID_BACKEND(id))
+						if (id < 0 || !VALID_BACKEND(id))
 						{
+							pool_error("CopyDataRow: pool_get_id returns invalid id: %d", id);
 							exit(1);
 						}
 						if (pool_write(CONNECTION(backend, id), &kind, 1))
