@@ -118,7 +118,7 @@ void pool_set_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return;
 	}
 
-	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_set_node_to_be_sent: invalid node id:%d", node_id);
 		return;
@@ -140,7 +140,7 @@ void pool_unset_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return;
 	}
 
-	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_unset_node_to_be_sent: invalid node id:%d", node_id);
 		return;
@@ -228,7 +228,7 @@ bool pool_is_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return false;
 	}
 
-	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_is_node_to_be_sent: invalid node id:%d", node_id);
 		return false;
@@ -1241,11 +1241,10 @@ void where_to_send_deallocate(POOL_QUERY_CONTEXT *query_context, Node *node)
 	DeallocateStmt *d = (DeallocateStmt *)node;
 	POOL_SENT_MESSAGE *msg;
 
-	/* DELLOCATE ALL? */
+	/* DEALLOCATE ALL? */
 	if (d->name == NULL)
 	{
 		pool_setall_node_to_be_sent(query_context);
-		return;
 	}
 	else
 	{
@@ -1257,11 +1256,11 @@ void where_to_send_deallocate(POOL_QUERY_CONTEXT *query_context, Node *node)
 			/* Inherit same map from PREPARE or PARSE */
 			pool_copy_prep_where(msg->query_context->where_to_send,
 								 query_context->where_to_send);
+			return;
 		}
-		return;
+		/* prepared statement was not found */
+		pool_setall_node_to_be_sent(query_context);
 	}
-	/* prepared statement was not found */
-	pool_setall_node_to_be_sent(query_context);
 }
 
 /*
